/*
 * Copyright (c) 2016 Manuel Braga <mul.braga@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <libavformat/avformat.h>
#include "common.h"

static AVFormatContext	*context = NULL;
static int		stream_id = -1;
static AVPacket		packet;

AVStream* demux_open(char *filename)
{
	av_register_all();

	if(avformat_open_input(&context, filename, NULL, NULL) < 0) {
		fprintf(stderr, "Could not open: %s\n", filename);
		exit(1);
	}

	if(avformat_find_stream_info(context, NULL) < 0) {
		fprintf(stderr, "Could not find stream info.\n");
		exit(1);
	}

	av_dump_format(context, 0, filename, 0);

	AVCodec *codec = NULL;
	stream_id = av_find_best_stream(context, AVMEDIA_TYPE_VIDEO, -1, -1, &codec, 0);
	if(stream_id < 0) {
		fprintf(stderr, "Could not find best stream.\n");
		exit(1);
	}
	AVStream *stream = context->streams[stream_id];
	printf("stream %d: frame %d: %dx%d  %d(%d) %s\n", stream_id, stream->codec->frame_number,
					   stream->codec->width, stream->codec->height,
					   stream->codec->codec_id, CODEC_ID_VP8,
					   codec->name);
	return stream;
}

int demux_packet_dump(uint8_t **data, int *size)
{
	int ok = 0;
	av_free_packet(&packet);
	av_init_packet(&packet);
	packet.data = NULL;
	packet.size = 0;

	ok = av_read_frame(context, &packet);

#if 1
	/*printf("packet size: %d %ld\n", packet.size, packet.dts);*/
	/*av_hex_dump(stdout, packet.data, packet.size);*/
	if(ok == 0)
		av_pkt_dump2(stdout, &packet, 1, context->streams[packet.stream_index]);
#endif
	*data = packet.data;
	*size = packet.size;

	return ok == 0;
}

AVPacket* demux_packet(void)
{
	av_free_packet(&packet);
	av_init_packet(&packet);
	packet.data = NULL;
	packet.size = 0;

	while(av_read_frame(context, &packet) >= 0) {
		if(packet.stream_index == stream_id)
			return &packet;
	}
	
	return NULL;
}


void demux_close(void)
{
	av_free_packet(&packet);
	avformat_close_input(&context);
}

